                ORG	$3f00
; *****************************************************************************
; * The Music Studio Player Engine
; *
; * Based on code written by Sasa Pusica for the utility, The Music Studio.
; * Modified by Chris Cowley
; *
; * Produced by Beepola v1.06.01
; *
; *
; *
; * Z80 to 6809 conversion by John Kowalski (Sock Master)
; *
; * Original Z80 to 6809 conversion idea and debugging by Simon Jonassen
; *
; * 6809 code changes and optimization by Steve Evans (Zephyr)
; * http://archive.worldofdragon.org/
; ******************************************************************************

start		pshs	dp,cc
		orcc	#$50			; Disable IRQ/FIRQ interrupts
		lda	#$3f			; Set DP = $3F
		tfr	a,dp
		setdp	$3f

		ldd	#PATTERNDATA 		; Get start address of pattern data
		addd	MUSICDATA		; Add loop start point to the pattern data start address
		tfr	d,x			; Transfer pattern data loop start address to the X register

		ldd	#PATTERNDATA 		; Get start address of pattern data
		addd	MUSICDATA+2		; Add song length to the pattern data loop start address
		std	<patloopend+1		; Set pattern data loop end address

		jsr	<audio_on		; Select DAC as sound source and enable audio
		sts	stack_pointer+2		; Save system stack pointer address
		lds	#$ff20

; ********************************************************************************************************
; * NEXT_PATTERN
; *
; * Select the next pattern in sequence (and handle looping if we've reached PATTERN_LOOP_END
; * Execution falls through to PLAYNOTE to play the first note from our next pattern
; ********************************************************************************************************
nextpat
		lda	[,x]			; X = Pattern data pointer 
		sta	<qtempo+1
		ldu	#1
playnote 
		ldy	,x			; Y = Start of note data
		tfr	u,d
		ldd	d,y			; D,Y = Note data pointer
		cmpa	#$fe			; $FE indicates end of pattern
		bne	continue

		leax	2,x
patloopend	cmpx	#0000			; Check for end of pattern loop
		blo	nextpat

		jmp	<audio_off		; Disable audio and return to basic

continue	sta	<ch1freq+1
		stb	<ch2freq+1
noteptr		leau	2,u			; Increment the note pointer by 2 (one note per chan)

qtempo		ldd	#0000			; A = Tempo | B = 0
		std	<tempc
		stb	<bord1+1
		stb	<bord2+1		; So now tempb = 0, tempc = Tempo | bord1 & bord2 = 0

outputnote
		ldb	<ch1freq+1		; Put note frequency for chan 1 into IXH
		stb	<ch1ix+1
		stb	<ch1count
		decb
		stb	<ltemp1
		beq	continue1
		ldb	#$40
continue1	stb	<xore1+1
		stb	<xore1b+1

		ldb	<ch2freq+1		; Put note frequency for chan 2 into IXL
		stb	<ch2ix+1
		stb	<ch2count
		decb
		stb	<ltemp2
		beq	continue2
		ldb	#$40
continue2	stb	<xore2+1
		stb	<xore2b+1
continue3
bord1		lda	#00
		dec	<ch1count		; Dec H, which also holds the frequency value
		bne	l8055
xore1		eora	#00
		sta	<bord1+1
ch1freq		ldb	#00
		stb	<ch1count

ch1ix		ldb	#00
		cmpb	#$20
		bcc	l8055			; if B > $20 then this is not a drum effect, skip the INC D
		inc	<ch1freq+1		; create the "fast falling pitch" percussion effect
l8055		dec	<ltemp1
		bne	bord2
xore1b		eora	#00
		sta	<bord1+1
		ldb	<ch1freq+1
		decb
		stb	<ltemp1

bord2		adda	#00
		anda	#$c0			; Bits 6 and 7 are used to emulate the 2 channel sound
		sta	,s
		lda	<bord2+1
		dec	<ch2count
		bne	l806d
xore2		eora	#00
		sta	<bord2+1
ch2freq		ldb	#00
		stb	<ch2count
ch2ix		ldb	#00
		cmpb	#$20
		bcc	l806d			; if A > $20 then this is not a drum effect, skip the INC D
		inc	<ch2freq+1		; create the "fast falling pitch" percussion effect

l806d		dec	<ltemp2
		bne	l8073
xore2b		eora	#00
		sta	<bord2+1
		ldb	<ch2freq+1
		decb
		stb	<ltemp2
l8073		dec	<tempb
		bne	continue3
		dec	<tempc
		bne	continue3
		jmp	<playnote

tempc		fcb	0
tempb		fcb	0
ch1count	fdb	0
ch2count	fdb	0
ltemp1		fcb	0
ltemp2		fcb	0
DOUBLE_SPEED	fcb	0			; 0 = Normal CPU speed, 1 = Double CPU speed 

audio_on
		lda	$ff01			; Select DAC as sound source
		anda	#$f7
		sta	$ff01
		lda	$ff03
		anda	#$f7
		sta	$ff03

		lda	$ff23			; Enable sound
		ora	#$8
		sta	$ff23	

		lda	DOUBLE_SPEED		; Check double speed flag
		beq	music
		sta	$ffd7			; Enable CPU double speed
		sta	$ffd9
music		rts

audio_off
		lda	$ff23 			; Disable sound 
		anda	#$f7
		sta	$ff23	

		lda	DOUBLE_SPEED		; Check double speed flag
		beq	stack_pointer
		sta	$ffd8			; Disable CPU double speed
		sta	$ffd6
stack_pointer	lds	#0000			; Restore system stack pointer address
		puls	dp,cc,pc		; Restore CC (Enable FIRQ/IRQ interrupts), DP, and return to basic

; ** Change the following LOOP START POINT and SONG LENGTH to DOUBLE BYTE VALUES (fdb) for this 16-bit engine **


; *** DATA ***
BORDER_COL                EQU $0
TEMPO                     fcb 242

MUSICDATA 
                    fdb 0    ; Loop start point * 2
                    fdb 272  ; Song Length * 2
PATTERNDATA         fdb      PAT0
                    fdb      PAT1
                    fdb      PAT2
                    fdb      PAT3
                    fdb      PAT4
                    fdb      PAT5
                    fdb      PAT6
                    fdb      PAT6
                    fdb      PAT7
                    fdb      PAT8
                    fdb      PAT9
                    fdb      PAT9
                    fdb      PAT10
                    fdb      PAT11
                    fdb      PAT12
                    fdb      PAT13
                    fdb      PAT14

	            fdb      PAT0
                    fdb      PAT1
                    fdb      PAT2
                    fdb      PAT3
                    fdb      PAT4
                    fdb      PAT5
                    fdb      PAT6
                    fdb      PAT6
                    fdb      PAT7
                    fdb      PAT8
                    fdb      PAT9
                    fdb      PAT9
                    fdb      PAT10
                    fdb      PAT11
                    fdb      PAT12
                    fdb      PAT13
                    fdb      PAT14

	            fdb      PAT0
                    fdb      PAT1
                    fdb      PAT2
                    fdb      PAT3
                    fdb      PAT4
                    fdb      PAT5
                    fdb      PAT6
                    fdb      PAT6
                    fdb      PAT7
                    fdb      PAT8
                    fdb      PAT9
                    fdb      PAT9
                    fdb      PAT10
                    fdb      PAT11
                    fdb      PAT12
                    fdb      PAT13
                    fdb      PAT14

	            fdb      PAT0
                    fdb      PAT1
                    fdb      PAT2
                    fdb      PAT3
                    fdb      PAT4
                    fdb      PAT5
                    fdb      PAT6
                    fdb      PAT6
                    fdb      PAT7
                    fdb      PAT8
                    fdb      PAT9
                    fdb      PAT9
                    fdb      PAT10
                    fdb      PAT11
                    fdb      PAT12
                    fdb      PAT13
                    fdb      PAT14

	            fdb      PAT0
                    fdb      PAT1
                    fdb      PAT2
                    fdb      PAT3
                    fdb      PAT4
                    fdb      PAT5
                    fdb      PAT6
                    fdb      PAT6
                    fdb      PAT7
                    fdb      PAT8
                    fdb      PAT9
                    fdb      PAT9
                    fdb      PAT10
                    fdb      PAT11
                    fdb      PAT12
                    fdb      PAT13
                    fdb      PAT14

	            fdb      PAT0
                    fdb      PAT1
                    fdb      PAT2
                    fdb      PAT3
                    fdb      PAT4
                    fdb      PAT5
                    fdb      PAT6
                    fdb      PAT6
                    fdb      PAT7
                    fdb      PAT8
                    fdb      PAT9
                    fdb      PAT9
                    fdb      PAT10
                    fdb      PAT11
                    fdb      PAT12
                    fdb      PAT13
                    fdb      PAT14

	            fdb      PAT0
                    fdb      PAT1
                    fdb      PAT2
                    fdb      PAT3
                    fdb      PAT4
                    fdb      PAT5
                    fdb      PAT6
                    fdb      PAT6
                    fdb      PAT7
                    fdb      PAT8
                    fdb      PAT9
                    fdb      PAT9
                    fdb      PAT10
                    fdb      PAT11
                    fdb      PAT12
                    fdb      PAT13
                    fdb      PAT14

	            fdb      PAT0
                    fdb      PAT1
                    fdb      PAT2
                    fdb      PAT3
                    fdb      PAT4
                    fdb      PAT5
                    fdb      PAT6
                    fdb      PAT6
                    fdb      PAT7
                    fdb      PAT8
                    fdb      PAT9
                    fdb      PAT9
                    fdb      PAT10
                    fdb      PAT11
                    fdb      PAT12
                    fdb      PAT13
                    fdb      PAT14

; *** Pattern data consists of pairs of frequency values CH1,CH2 with a single $FE to
; *** Mark the end of the pattern, and $01 for a rest
PAT0 
         fcb 8  ; Pattern tempo
             fcb 76,16
             fcb 114,227
             fcb 1,0
             fcb 114,227
             fcb 1,227
             fcb 1,227
             fcb 114,0
             fcb 1,227
             fcb 1,192
             fcb 96,192
             fcb 1,0
             fcb 1,192
             fcb 86,171
             fcb 1,171
             fcb 76,0
             fcb 1,171
         fcb $FE
PAT1 
         fcb 8  ; Pattern tempo
             fcb 1,16
             fcb 114,227
             fcb 1,0
             fcb 1,227
             fcb 114,227
             fcb 1,227
             fcb 114,0
             fcb 1,227
             fcb 1,192
             fcb 96,192
             fcb 1,0
             fcb 1,192
             fcb 114,171
             fcb 1,171
             fcb 128,0
             fcb 121,171
         fcb $FE
PAT2 
         fcb 8  ; Pattern tempo
             fcb 1,16
             fcb 86,227
             fcb 1,0
             fcb 1,227
             fcb 86,227
             fcb 1,227
             fcb 76,0
             fcb 1,227
             fcb 1,192
             fcb 114,192
             fcb 1,0
             fcb 1,192
             fcb 114,171
             fcb 1,171
             fcb 128,0
             fcb 121,171
         fcb $FE
PAT3 
         fcb 8  ; Pattern tempo
             fcb 1,16
             fcb 76,227
             fcb 1,0
             fcb 1,227
             fcb 76,227
             fcb 1,227
             fcb 76,0
             fcb 1,227
             fcb 1,192
             fcb 86,192
             fcb 1,0
             fcb 1,192
             fcb 96,171
             fcb 114,171
             fcb 128,0
             fcb 114,171
         fcb $FE
PAT4 
         fcb 8  ; Pattern tempo
             fcb 1,16
             fcb 227,30
             fcb 1,0
             fcb 227,30
             fcb 227,1
             fcb 227,1
             fcb 1,30
             fcb 227,1
             fcb 192,30
             fcb 192,1
             fcb 1,0
             fcb 192,30
             fcb 171,1
             fcb 171,1
             fcb 1,30
             fcb 171,1
         fcb $FE
PAT5 
         fcb 8  ; Pattern tempo
             fcb 1,16
             fcb 227,30
             fcb 1,0
             fcb 227,1
             fcb 227,30
             fcb 227,1
             fcb 1,0
             fcb 227,30
             fcb 192,1
             fcb 192,30
             fcb 1,30
             fcb 192,1
             fcb 171,30
             fcb 171,30
             fcb 1,30
             fcb 171,30
         fcb $FE
PAT6 
         fcb 8  ; Pattern tempo
             fcb 114,16
             fcb 227,128
             fcb 114,0
             fcb 227,128
             fcb 227,114
             fcb 227,128
             fcb 96,0
             fcb 227,1
             fcb 192,114
             fcb 192,128
             fcb 114,0
             fcb 192,128
             fcb 171,114
             fcb 171,128
             fcb 1,0
             fcb 171,1
         fcb $FE
PAT7 
         fcb 8  ; Pattern tempo
             fcb 76,16
             fcb 86,227
             fcb 76,0
             fcb 86,227
             fcb 76,227
             fcb 86,227
             fcb 96,0
             fcb 1,227
             fcb 86,192
             fcb 96,192
             fcb 114,0
             fcb 128,192
             fcb 114,171
             fcb 96,171
             fcb 86,0
             fcb 1,171
         fcb $FE
PAT8 
         fcb 8  ; Pattern tempo
             fcb 76,16
             fcb 86,227
             fcb 76,0
             fcb 86,227
             fcb 76,227
             fcb 86,227
             fcb 96,0
             fcb 1,227
             fcb 128,192
             fcb 114,192
             fcb 114,0
             fcb 128,192
             fcb 114,171
             fcb 128,171
             fcb 114,0
             fcb 1,171
         fcb $FE
PAT9 
         fcb 8  ; Pattern tempo
             fcb 128,16
             fcb 121,227
             fcb 114,0
             fcb 114,227
             fcb 1,227
             fcb 114,227
             fcb 114,0
             fcb 1,227
             fcb 128,192
             fcb 121,192
             fcb 114,0
             fcb 114,192
             fcb 1,171
             fcb 114,171
             fcb 114,0
             fcb 1,171
         fcb $FE
PAT10 
         fcb 8  ; Pattern tempo
             fcb 128,16
             fcb 121,227
             fcb 38,0
             fcb 43,227
             fcb 57,227
             fcb 64,227
             fcb 57,0
             fcb 64,227
             fcb 57,192
             fcb 43,192
             fcb 38,0
             fcb 43,192
             fcb 57,171
             fcb 64,171
             fcb 57,0
             fcb 64,171
         fcb $FE
PAT11 
         fcb 8  ; Pattern tempo
             fcb 64,16
             fcb 64,227
             fcb 38,0
             fcb 43,227
             fcb 57,227
             fcb 64,227
             fcb 57,0
             fcb 64,227
             fcb 57,192
             fcb 43,192
             fcb 38,0
             fcb 43,192
             fcb 57,171
             fcb 64,171
             fcb 57,0
             fcb 64,171
         fcb $FE
PAT12 
         fcb 8  ; Pattern tempo
             fcb 64,16
             fcb 64,227
             fcb 1,0
             fcb 57,227
             fcb 57,227
             fcb 1,227
             fcb 57,0
             fcb 57,227
             fcb 1,192
             fcb 1,192
             fcb 48,0
             fcb 1,192
             fcb 1,171
             fcb 48,171
             fcb 1,0
             fcb 1,171
         fcb $FE
PAT13 
         fcb 8  ; Pattern tempo
             fcb 48,16
             fcb 1,227
             fcb 1,0
             fcb 227,30
             fcb 227,1
             fcb 227,30
             fcb 1,0
             fcb 227,30
             fcb 192,1
             fcb 192,30
             fcb 1,0
             fcb 192,30
             fcb 171,30
             fcb 171,1
             fcb 1,30
             fcb 171,30
         fcb $FE
PAT14 
         fcb 8  ; Pattern tempo
             fcb 1,30
             fcb 227,30
             fcb 1,0
             fcb 227,1
         fcb $FE
